import { __assign } from "tslib";
import throttle from 'lodash.throttle';
import React, { useEffect, useMemo, useState } from 'react';
import { ChannelPreviewMessenger } from './ChannelPreviewMessenger';
import { useIsChannelMuted } from './hooks/useIsChannelMuted';
import { useChannelPreviewInfo } from './hooks/useChannelPreviewInfo';
import { getLatestMessagePreview as defaultGetLatestMessagePreview } from './utils';
import { useChatContext } from '../../context/ChatContext';
import { useTranslationContext } from '../../context/TranslationContext';
import { useMessageDeliveryStatus } from './hooks/useMessageDeliveryStatus';
export var ChannelPreview = function (props) {
    var channel = props.channel, _a = props.Preview, Preview = _a === void 0 ? ChannelPreviewMessenger : _a, channelUpdateCount = props.channelUpdateCount, _b = props.getLatestMessagePreview, getLatestMessagePreview = _b === void 0 ? defaultGetLatestMessagePreview : _b;
    var _c = useChatContext('ChannelPreview'), activeChannel = _c.channel, client = _c.client, setActiveChannel = _c.setActiveChannel;
    var _d = useTranslationContext('ChannelPreview'), t = _d.t, userLanguage = _d.userLanguage;
    var _e = useChannelPreviewInfo({ channel: channel }), displayImage = _e.displayImage, displayTitle = _e.displayTitle;
    var _f = useState(channel.state.messages[channel.state.messages.length - 1]), lastMessage = _f[0], setLastMessage = _f[1];
    var _g = useState(0), unread = _g[0], setUnread = _g[1];
    var messageDeliveryStatus = useMessageDeliveryStatus({
        channel: channel,
        lastMessage: lastMessage,
    }).messageDeliveryStatus;
    var isActive = (activeChannel === null || activeChannel === void 0 ? void 0 : activeChannel.cid) === channel.cid;
    var muted = useIsChannelMuted(channel).muted;
    useEffect(function () {
        var handleEvent = function (event) {
            if (!event.cid)
                return setUnread(0);
            if (channel.cid === event.cid)
                setUnread(0);
        };
        client.on('notification.mark_read', handleEvent);
        return function () { return client.off('notification.mark_read', handleEvent); };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    useEffect(function () {
        var handleEvent = function (event) {
            var _a, _b;
            if (channel.cid !== event.cid)
                return;
            if (((_a = event.user) === null || _a === void 0 ? void 0 : _a.id) !== ((_b = client.user) === null || _b === void 0 ? void 0 : _b.id))
                return;
            setUnread(channel.countUnread());
        };
        channel.on('notification.mark_unread', handleEvent);
        return function () {
            channel.off('notification.mark_unread', handleEvent);
        };
    }, [channel, client]);
    var refreshUnreadCount = useMemo(function () {
        return throttle(function () {
            if (muted) {
                setUnread(0);
            }
            else {
                setUnread(channel.countUnread());
            }
        }, 400);
    }, [channel, muted]);
    useEffect(function () {
        refreshUnreadCount();
        var handleEvent = function () {
            setLastMessage(channel.state.latestMessages[channel.state.latestMessages.length - 1]);
            refreshUnreadCount();
        };
        channel.on('message.new', handleEvent);
        channel.on('message.updated', handleEvent);
        channel.on('message.deleted', handleEvent);
        channel.on('message.undeleted', handleEvent);
        channel.on('channel.truncated', handleEvent);
        return function () {
            channel.off('message.new', handleEvent);
            channel.off('message.updated', handleEvent);
            channel.off('message.deleted', handleEvent);
            channel.off('message.undeleted', handleEvent);
            channel.off('channel.truncated', handleEvent);
        };
    }, [channel, refreshUnreadCount, channelUpdateCount]);
    if (!Preview)
        return null;
    var latestMessagePreview = getLatestMessagePreview(channel, t, userLanguage);
    return (React.createElement(Preview, __assign({}, props, { active: isActive, displayImage: displayImage, displayTitle: displayTitle, lastMessage: lastMessage, latestMessage: latestMessagePreview, latestMessagePreview: latestMessagePreview, messageDeliveryStatus: messageDeliveryStatus, setActiveChannel: setActiveChannel, unread: unread })));
};
